package ink.wgink.interfaces.department;

import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.department.DepartmentDTO;
import ink.wgink.pojo.pos.DepartmentPO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IDepartmentService
 * @Description: 组织机构
 * @Author: WangGeng
 * @Date: 2021/1/24 12:53
 * @Version: 1.0
 **/
public interface IDepartmentBaseService {

    /**
     * 部门列表
     *
     * @param params
     * @return
     */
    List<DepartmentDTO> list(Map<String, Object> params);

    /**
     * 通过ID获取组织部门详情
     *
     * @param departmentId
     * @return
     */
    DepartmentDTO get(String departmentId);

    /**
     * easyui组织部门
     *
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPage(ListPage page);

    /**
     * easyui组织部门列表
     *
     * @param departmentParentId
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageByParentId(String departmentParentId, ListPage page);

    /**
     * 组织部门列表，递归获取全部内容
     *
     * @param params
     * @return
     */
    List<DepartmentDTO> listAll(Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param departmentParentId
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(String departmentParentId, Map<String, Object> params);

    /**
     * 部门ID列表
     *
     * @param params
     * @return
     */
    List<String> listId(Map<String, Object> params);

    /**
     * 获取基础部门ID列表
     *
     * @param departmentPOS
     * @return
     */
    List<String> listRootId(List<DepartmentPO> departmentPOS);

    /**
     * 获取上级部门
     *
     * @param departmentPO1
     * @param departmentPO2
     * @return
     */
    DepartmentPO getHigherLevel(DepartmentPO departmentPO1, DepartmentPO departmentPO2);

    /**
     * 获得同一级部门数量（按部门名称）
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 通过ID获取组织部门详情（简单格式）
     *
     * @param departmentId
     * @return
     */
    DepartmentPO getPO(String departmentId);

    /**
     * 部门列表
     *
     * @param params
     * @return
     */
    List<DepartmentPO> listPO(Map<String, Object> params);

    /**
     * 部门列表（简单格式）
     *
     * @param departmentIds 部门ID列表
     * @return
     */
    List<DepartmentPO> listPO(List<String> departmentIds);

    /**
     * 通过上级ID获取组织部门全部列表（简单格式）
     *
     * @param departmentParentId
     * @return
     */
    List<DepartmentPO> listByParentId(String departmentParentId);

    /**
     * 通过上级ID和添加获取组织部门全部增量列表（简单格式）
     *
     * @param departmentParentId
     * @param addDate
     * @return
     */
    List<DepartmentPO> listByParentIdAndAddDate(String departmentParentId, String addDate);

    /**
     * 通过上级ID和更新获取组织部门全部新增和修改的列表（简单格式）
     *
     * @param departmentParentId
     * @param updateDate
     * @return
     */
    List<DepartmentPO> listByParentIdAndUpdateDate(String departmentParentId, String updateDate);

}
