package ink.wgink.common.base;

import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.menu.IMenuBaseService;
import ink.wgink.interfaces.user.IUserCheckService;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.pojo.dtos.permission.SystemApiDTO;
import ink.wgink.util.RegexUtil;
import io.swagger.models.Operation;
import io.swagger.models.Path;
import io.swagger.models.Swagger;
import io.swagger.models.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import springfox.documentation.service.Documentation;
import springfox.documentation.spring.web.DocumentationCache;
import springfox.documentation.swagger2.mappers.ServiceModelToSwagger2Mapper;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IndexController
 * @Description: index
 * @Author: WangGeng
 * @Date: 2021/2/24 20:17
 * @Version: 1.0
 **/
@Controller
public class ApiInfoController {

    private static String API_TYPE_GET = "get";
    private static String API_TYPE_POST = "post";
    private static String API_TYPE_PUT = "put";
    private static String API_TYPE_DELETE = "delete";

    @Autowired
    private DocumentationCache documentationCache;
    @Autowired
    private ServiceModelToSwagger2Mapper serviceModelToSwagger2Mapper;

    @GetMapping("system-api/{group}/{apiType}")
    @ResponseBody
    public SystemApiDTO getSystemApi(@PathVariable("group") String group, @PathVariable("apiType") String apiType, HttpServletRequest request) {
        boolean apiTypeFlag = StringUtils.equalsIgnoreCase(API_TYPE_POST, apiType) ||
                StringUtils.equalsIgnoreCase(API_TYPE_DELETE, apiType) ||
                StringUtils.equalsIgnoreCase(API_TYPE_PUT, apiType) ||
                StringUtils.equalsIgnoreCase(API_TYPE_GET, apiType);
        if (!apiTypeFlag) {
            throw new ParamsException("api类型错误，只能是 post、delete、put、get 其中一种");
        }
        Documentation documentation = documentationCache.documentationByGroup(group);
        if (documentation == null) {
            throw new SearchException("无任何接口");
        }
        Swagger swagger = serviceModelToSwagger2Mapper.mapDocumentation(documentation);
        // 标签
        List<Tag> tags = swagger.getTags();
        List<SystemApiDTO.ApiDTO> apis = new ArrayList<>();
        for (Map.Entry<String, Path> kv : swagger.getPaths().entrySet()) {
            String keyPath = kv.getKey();
            Path valuePath = kv.getValue();
            Operation operation = null;
            if (StringUtils.equals(API_TYPE_POST, apiType)) {
                operation = valuePath.getPost();
            } else if (StringUtils.equals(API_TYPE_DELETE, apiType)) {
                operation = valuePath.getDelete();
            } else if (StringUtils.equals(API_TYPE_PUT, apiType)) {
                operation = valuePath.getPut();
            } else if (StringUtils.equals(API_TYPE_GET, apiType)) {
                operation = valuePath.getGet();
            }
            if (operation == null) {
                continue;
            }
            String apiTag = null;
            for (Tag tag : tags) {
                if (StringUtils.equals(tag.getName(), operation.getTags().get(0))) {
                    apiTag = tag.getDescription().replaceAll("\\s", "");
                }
            }
            if (apiTag == null) {
                continue;
            }
            apis.add(new SystemApiDTO.ApiDTO(RegexUtil.replacePathParams(keyPath, "*"), operation.getSummary(), apiTag));
        }
        SystemApiDTO systemApiDTO = new SystemApiDTO();
        systemApiDTO.setSystemTag(request.getContextPath());
        systemApiDTO.setApis(apis);
        return systemApiDTO;
    }
}
