package ink.wgink.login.base.controller.route;

import ink.wgink.common.component.SecurityComponent;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.menu.IMenuBaseService;
import ink.wgink.interfaces.role.IRoleMenuBaseService;
import ink.wgink.login.base.consts.IUserCenterConst;
import ink.wgink.login.base.manager.ConfigManager;
import ink.wgink.pojo.bos.UserInfoBO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.pojo.pos.RolePO;
import ink.wgink.properties.ServerProperties;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IndexRouteController
 * @Description: index
 * @Author: wanggeng
 * @Date: 2021/2/27 3:38 下午
 * @Version: 1.0
 */
@Controller
public class IndexRouteController {

    @Autowired
    private SecurityComponent securityComponent;
    @Autowired
    private ServerProperties serverProperties;
    @Autowired(required = false)
    private IMenuBaseService menuBaseService;
    @Autowired(required = false)
    private IRoleMenuBaseService roleMenuBaseService;

    @GetMapping("index")
    public ModelAndView goIndex() {
        if (!StringUtils.isBlank(serverProperties.getDefaultIndexPage())) {
            return new ModelAndView(new RedirectView(serverProperties.getDefaultIndexPage()));
        }
        return new ModelAndView("forward:/default-main");
    }

    /**
     * 默认主页
     *
     * @return
     */
    @GetMapping("default-main")
    public ModelAndView defaultMain() {
        ModelAndView mv = new ModelAndView("default-main");
        UserInfoBO userInfoBO = securityComponent.getCurrentUser();
        mv.addObject("userUsername", userInfoBO.getUserUsername());
        Map<String, Object> config = ConfigManager.getInstance().getConfig();
        if (!Objects.isNull(config.get(IUserCenterConst.SYSTEM_TITLE)) && !StringUtils.isBlank(config.get(IUserCenterConst.SYSTEM_TITLE).toString())) {
            mv.addObject(IUserCenterConst.SYSTEM_TITLE, config.get(IUserCenterConst.SYSTEM_TITLE).toString());
        } else {
            mv.addObject(IUserCenterConst.SYSTEM_TITLE, serverProperties.getSystemTitle());
        }
        if (!Objects.isNull(config.get(IUserCenterConst.SYSTEM_SUB_TITLE)) && !StringUtils.isBlank(config.get(IUserCenterConst.SYSTEM_SUB_TITLE).toString())) {
            mv.addObject(IUserCenterConst.SYSTEM_SUB_TITLE, config.get(IUserCenterConst.SYSTEM_SUB_TITLE).toString());
        } else {
            mv.addObject(IUserCenterConst.SYSTEM_SUB_TITLE, serverProperties.getSystemSubTitle());
        }
        // 系统LOGO
        if (!Objects.isNull(config.get(IUserCenterConst.SYSTEM_LOGO)) && !StringUtils.isBlank(config.get(IUserCenterConst.SYSTEM_LOGO).toString())) {
            mv.addObject(IUserCenterConst.SYSTEM_LOGO, config.get(IUserCenterConst.SYSTEM_LOGO).toString());
        } else {
            mv.addObject(IUserCenterConst.SYSTEM_LOGO, "");
        }
        mv.addObject("ws", serverProperties.getWs());
        // 菜单模式
        if (!Objects.isNull(config.get(IUserCenterConst.MENU_MODE)) && !StringUtils.isBlank(config.get(IUserCenterConst.MENU_MODE).toString())) {
            mv.addObject(IUserCenterConst.MENU_MODE, config.get(IUserCenterConst.MENU_MODE).toString());
        }
        if (menuBaseService != null) {
            List<MenuDTO> menus;
            if (StringUtils.equalsIgnoreCase(ISystemConstant.ADMIN, userInfoBO.getUserUsername())) {
                // 管理员
                List<String> menuIds = roleMenuBaseService.listMenuId(ISystemConstant.ADMIN);
                if (menuIds.isEmpty()) {
                    menus = menuBaseService.listAllByParentId(IMenuBaseService.MENU_UNIFIED_USER);
                } else {
                    menus = menuBaseService.listAllByParentIdAndIds(IMenuBaseService.MENU_UNIFIED_USER, menuIds);
                }
            } else {
                // 普通用户
                List<RolePO> rolePOs = securityComponent.getCurrentUser().getRoles();
                if (rolePOs.isEmpty()) {
                    menus = new ArrayList<>();
                } else {
                    List<String> roleIds = new ArrayList<>();
                    for (RolePO rolePO : rolePOs) {
                        roleIds.add(rolePO.getRoleId());
                    }
                    List<String> menuIds = roleMenuBaseService.listMenuId(roleIds);
                    menus = menuBaseService.listAllByParentIdAndIds(IMenuBaseService.MENU_UNIFIED_USER, menuIds);
                }
            }
            mv.addObject("menus", menus);
        }
        return mv;
    }

    /**
     * 默认导航首页
     *
     * @return
     */
    @GetMapping("default-home")
    public ModelAndView defaultHome() {
        ModelAndView mv;
        if (!StringUtils.isBlank(serverProperties.getDefaultHomePage())) {
            mv = new ModelAndView(new RedirectView(serverProperties.getDefaultHomePage()));
        } else {
            mv = new ModelAndView("default-home");
        }
        return mv;
    }

}
