package ink.wgink.login.wechat.service.sign.impl;

import com.alibaba.fastjson.JSONObject;
import ink.wgink.exceptions.PropertiesException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.WechatAccessTokenForUserException;
import ink.wgink.exceptions.WechatUserInfoException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.login.base.service.BaseAppSignService;
import ink.wgink.module.wechat.pojo.bos.miniapp.MiniAppUserInfoBO;
import ink.wgink.module.wechat.service.miniapp.IMiniAppUserService;
import ink.wgink.properties.wechat.miniapp.MiniAppProperties;
import ink.wgink.login.wechat.pojo.vos.sign.MiniAppLoginVO;
import ink.wgink.login.wechat.service.sign.IMiniAppSignService;
import ink.wgink.service.user.pojo.pos.UserPO;
import ink.wgink.service.user.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: MiniAppSignServiceImpl
 * @Description: 小程序登录
 * @Author: wanggeng
 * @Date: 2021/4/8 5:14 下午
 * @Version: 1.0
 */
@Service
public class MiniAppSignServiceImpl extends BaseAppSignService implements IMiniAppSignService {

    @Autowired
    private MiniAppProperties miniAppProperties;
    @Autowired
    private IMiniAppUserService miniAppUserService;
    @Autowired
    private IUserService userService;

    @Override
    public String defaultSign(MiniAppLoginVO miniAppLoginVO) throws Exception {
        if (miniAppLoginVO == null) {
            throw new PropertiesException("未有相关配置");
        }
        Map<String, Object> params = new HashMap<>(2);
        params.put("appid", miniAppProperties.getAppKey());
        params.put("secret", miniAppProperties.getAppSecret());
        params.put("js_code", miniAppLoginVO.getJsCode());
        params.put("grant_type", miniAppProperties.getGrantType());

        StringBuilder url = new StringBuilder(miniAppProperties.getAuthorizeUrl());
        url.append("?appid={appid}&secret={secret}&js_code={js_code}&grant_type={grant_type}");
        RestTemplate restTemplate = new RestTemplate();

        String resultJson = restTemplate.getForObject(url.toString(), String.class, params);
        MiniAppUserInfoBO miniAppUserInfoBO = JSONObject.parseObject(resultJson, MiniAppUserInfoBO.class);
        if (miniAppUserInfoBO == null) {
            throw new WechatUserInfoException("获取微信用户信息失败");
        }
        if (miniAppUserInfoBO.getErrcode() != null && miniAppUserInfoBO.getErrcode() != 0) {
            throw new WechatAccessTokenForUserException(String.format("获取用户信息失败，错误码：%s，错误信息：%s", miniAppUserInfoBO.getErrcode(), miniAppUserInfoBO.getErrmsg()));
        }
        LOG.debug("绑定用户");
        String userId = miniAppUserService.createAndReturnUserId(miniAppProperties.getAppKey(), miniAppUserInfoBO.getOpenid());
        UserPO userPO = userService.getPO(userId);
        if (userPO == null) {
            throw new SearchException("用户不存在");
        }
        String isRandomUsername = ISystemConstant.IS_FALSE_INT;
        if (userPO.getUserUsername().startsWith(IMiniAppUserService.MINIAPP_RANDOM_USER_PREFIX)) {
            isRandomUsername = ISystemConstant.IS_TRUE_INT;
        }
        return getToken(userPO) + "_" + isRandomUsername;
    }
}
