package ink.wgink.module.wechat.service.miniapp.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.wechat.dao.miniapp.IMiniAppUserDao;
import ink.wgink.module.wechat.pojo.pos.miniapp.MiniAppUserPO;
import ink.wgink.module.wechat.service.miniapp.IMiniAppUserService;
import ink.wgink.service.user.pojo.vos.UserVO;
import ink.wgink.service.user.service.IUserService;
import ink.wgink.util.date.DateUtil;
import ink.wgink.util.string.WStringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: MiniAppUserServiceImpl
 * @Description: 小程序用户
 * @Author: wanggeng
 * @Date: 2021/4/8 5:38 下午
 * @Version: 1.0
 */
@Service
public class MiniAppUserServiceImpl extends DefaultBaseService implements IMiniAppUserService {

    @Value("${user.default-password:"+ ISystemConstant.DEFAULT_PASSWORD +"}")
    private String defaultPassword;
    @Autowired
    private IMiniAppUserDao miniAppUserDao;
    @Autowired
    private IUserService userService;

    @Override
    public String createAndReturnUserId(String appKey, String openId) {
        MiniAppUserPO miniAppUserPO = getPO(appKey, openId);
        if (miniAppUserPO != null) {
            LOG.debug("已经绑定小程序用户");
            return miniAppUserPO.getUserId();
        }
        // 随机字符串
        String randomUserName = WStringUtil.randomSubStr(openId, 6);
        UserVO userVO = new UserVO();
        userVO.setUserUsername(MINIAPP_RANDOM_USER_PREFIX + randomUserName);
        userVO.setUserName("微信M" + randomUserName);
        userVO.setUserPassword(defaultPassword);
        userVO.setUserState(0);
        userVO.setUserType(2);
        String userId = userService.saveAndReturnId(userVO, true);

        Map<String, Object> params = getHashMap(6);
        params.put("appId", appKey);
        params.put("openId", openId);
        params.put("userId", userId);
        params.put("isInitAccount", 1);
        params.put("gmtCreate", DateUtil.getTime());
        miniAppUserDao.save(params);
        return userId;
    }

    @Override
    public MiniAppUserPO getPO(Map<String, Object> params) {
        return miniAppUserDao.getPO(params);
    }

    @Override
    public MiniAppUserPO getPO(String userId) {
        Map<String, Object> params = getHashMap(2);
        params.put("userId", userId);
        return getPO(params);
    }

    @Override
    public MiniAppUserPO getPO(String appId, String openId) {
        Map<String, Object> params = getHashMap(4);
        params.put("appId", appId);
        params.put("openId", openId);
        return getPO(params);
    }
}
