package ink.wgink.module.wechat.service.official.account;

import ink.wgink.module.wechat.enums.XmlEventTypeEnum;
import ink.wgink.module.wechat.pojo.dtos.official.account.OfficialAccountUserDTO;
import ink.wgink.module.wechat.pojo.pos.official.account.OfficialAccountUserPO;
import ink.wgink.module.wechat.pojo.vos.official.account.OfficialAccountUserVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始,
 *
 * @ClassName: IOfficialAccountUserService
 * @Description: 公众号用户，通过关注公众号，自动添加用户账号和公众号账号的关联关系；业务上，如果涉及到小程序和公众号等多端，则在不绑定手机的情况下，为了统一业务，必须多端绑定手机，统一手机绑定统一账号。
 * @Author: wanggeng
 * @Date: 2021/4/28 6:15 下午
 * @Version: 1.0
 */
public interface IOfficialAccountUserService {

    /**
     * 公众号用户前缀
     */
    String OFFICIAL_ACCOUNT_RANDOM_USER_PREFIX = "WX_";

    /**
     * 新增用户，用户关注微信公众号时触执行，创建随机账号；如果已经有则，修改订阅状态
     *
     * @param officialAccountUserVO
     */
    void save(OfficialAccountUserVO officialAccountUserVO);

    /**
     * 修改状态，修改订阅状态
     *
     * @param openId
     * @param xmlEventTypeEnum
     */
    void updateStatus(String openId, XmlEventTypeEnum xmlEventTypeEnum);

    /**
     * 修改初始账号标识
     *
     * @param openId
     * @param isInitAccount
     */
    void updateIsInitAccount(String openId, int isInitAccount);

    /**
     * 详情
     *
     * @param params
     * @return
     */
    OfficialAccountUserDTO get(Map<String, Object> params);

    /**
     * 详情
     *
     * @param appId
     * @param openId
     * @return
     */
    OfficialAccountUserDTO get(String appId, String openId);

    /**
     * 详情
     *
     * @param params
     * @return
     */
    OfficialAccountUserPO getPO(Map<String, Object> params);

    /**
     * 详情
     *
     * @param appId
     * @param openId
     * @return
     */
    OfficialAccountUserPO getPO(String appId, String openId);

    /**
     * 列表
     *
     * @param params
     * @return
     */
    List<OfficialAccountUserDTO> list(Map<String, Object> params);

    /**
     * 列表
     *
     * @param params
     * @return
     */
    List<OfficialAccountUserPO> listPO(Map<String, Object> params);

    /**
     * 分页
     *
     * @param page
     * @return
     */
    SuccessResultList<List<OfficialAccountUserDTO>> listPage(ListPage page);

    /**
     * 获取用户码
     *
     * @param openId 公众号用户openId
     * @return
     */
    String getUserCodeByOpenId(String openId);

    /**
     * 用户码获取用户
     *
     * @param userCode
     * @return
     */
    OfficialAccountUserPO getPOByUserCode(String userCode);

}
