package ink.wgink.service.user.startup;

import ink.wgink.service.user.dao.IUserAdjustmentDao;
import ink.wgink.service.user.dao.IUserDao;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.util.date.DateUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: ServiceUserStartUp
 * @Description: 用户业务
 * @Author: wanggeng
 * @Date: 2021/2/5 8:59 下午
 * @Version: 1.0
 */
@Component
public class ServiceUserStartUp implements ApplicationRunner {
    private static final Logger LOG = LoggerFactory.getLogger(ServiceUserStartUp.class);

    @Autowired
    private IUserDao userDao;
    @Autowired
    private IUserAdjustmentDao userAdjustmentDao;
    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    public void run(ApplicationArguments args) throws Exception {
        initTable();
    }

    /**
     * 建表
     */
    private void initTable() {
        LOG.debug("创建 sys_user 表");
        userDao.createTable();
        LOG.debug("检查 admin 是否存在");
        Map<String, Object> params = new HashMap<>(2);
        params.put("userUsername", "admin");
        UserDTO userDTO = userDao.get(params);
        if (userDTO == null) {
            LOG.debug("admin 不存在，新增");
            String currentDate = DateUtil.getTime();
            params.put("userId", "1");
            params.put("userPassword", passwordEncoder.encode(DigestUtils.md5Hex(DigestUtils.md5Hex(DigestUtils.md5Hex("aaa111!!!")))));
            params.put("userName", "超级管理员");
            params.put("userType", 1);
            params.put("userState", 0);
            params.put("gmtCreate", currentDate);
            params.put("creator", "1");
            params.put("gmtModified", currentDate);
            params.put("modifier", "1");
            params.put("isDelete", 0);
            userDao.save(params);
        }

        LOG.debug("创建 sys_user_adjustment 表");
        userAdjustmentDao.createTable();
    }
}
